#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;
layout(location = 3) in ivec4 boneIds;
layout(location = 4) in vec4 weights;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

const int MAX_BONES = 100;
const int MAX_BONE_INFLUENCE = 4;
uniform mat4 finalBonesMatrices[MAX_BONES];

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

void main() {
	
    //vec4 totalPosition = vec4(position, 1.0f);
	vec4 totalPosition = vec4(0.0f);
    vec3 totalNormal = vec3(0.0f);
	
    for(int i = 0; i < MAX_BONE_INFLUENCE; i++)
    {
        if(boneIds[i] == -1) 
            continue;
        if(boneIds[i] >= MAX_BONES ) 
        {
            totalPosition = vec4(position, 1.0f);
            break;
        }
        vec4 localPosition = finalBonesMatrices[boneIds[i]] * vec4(position, 1.0f);
        totalPosition += localPosition * weights[i];
        
		vec3 localNormal = mat3(finalBonesMatrices[boneIds[i]]) * normal;
		totalNormal += localNormal * weights[i];
    }
	
	if (boneIds.x == -1 && boneIds.y == -1 && boneIds.z == -1 && boneIds.w == -1) {
		totalPosition = vec4(position, 1.0f);
	}
	
	
	
	
	
	
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
	//vec4 position_world_ = modelMatrix * vec4(position, 1);
	vec4 position_world_ = modelMatrix * totalPosition;
	vert.position_world = position_world_.xyz;
	gl_Position = viewProjMatrix * position_world_;
}